module ccm_numz
! basic real types
    integer, parameter:: b4 = selected_real_kind(5)
    integer, parameter:: b8 = selected_real_kind(10)
    integer, parameter:: c4 = selected_real_kind(5) 
    integer, parameter:: c8 = selected_real_kind(10)
    integer, parameter:: b16= selected_real_kind(20) 
    integer, parameter:: c16= selected_real_kind(20) 
    integer, parameter:: def_int = kind(1)
    integer, parameter:: alt_int = selected_int_kind(16) 
! other values used by the api
    integer mpi_err
    integer, parameter::MPI_ROOT=0
    integer, parameter::MPI_ZERO=0
    integer, parameter:: mydouble=1
    integer, parameter:: myint=2
    integer, parameter:: mylogical=3
    integer, parameter:: mycharacter=4
    integer, parameter:: myreal=5
    integer, parameter:: mycomp=6
    integer, parameter:: mydpcomp=7
    integer, parameter:: myquad=8
    integer, parameter:: myqpcomp=9
    integer,save :: mylong
    integer,save :: mycomm,myid,numnodes
    integer :: do_tests,test_ray1(8),test_ray2(8),test_ray3(8)
    logical :: passed_test
    type node_list
    	integer from,to
    	logical done
    end type node_list
    type(node_list),allocatable :: got_list(:)
! public values for the API
    integer, parameter :: ccm_alloff=0
    integer, parameter :: ccm_checksize=1
    integer, parameter :: ccm_deadlock=2
    integer, parameter :: ccm_trace=4
    integer, parameter :: ccm_internal=8
    integer, parameter :: ccm_reproducible=0
    integer, parameter :: ccm_fast=1
    character (len=12) :: ccm_name
    logical,save :: ccm_auto_print
    real(b8),save :: ccm_timeout
! error information
    character(len=128),save:: err_str0,err_str1,err_str2,err_str3,err_str4
! machine specific values
    integer :: ccm_iounits(1) = (/0/)
! if force_four is true then we will define
! our mpi_real in term of mpi_character
    logical, parameter :: force_four= .true.
    integer :: ccm_start_time(8)
	interface
		subroutine ccm_check_sizes(routine,test1,test2,test3,root,passed)
			character(len=*) :: routine
			integer :: test1(:),test2(:),test3(:)
			integer :: root
			logical :: passed
		end subroutine
	end interface
! start of stuff unique to shmem
	integer, parameter :: maxprocs=64
	integer, parameter :: bsize8=10240
	integer, parameter :: csize8=10240
	integer, parameter :: bsize4=10240
	integer, parameter :: csize4=10240
	integer, parameter :: isize4=10240
	integer, parameter :: lsize4=10240
	integer, parameter :: asize4=10240
	
	complex(c16) :: qpcomp_buf(csize8)
	common /ccm_cx16/ qpcomp_buf
	
	real(b16) :: qp_buf(bsize8)
	common /ccm_rx16/ qp_buf
	
	complex(c8) :: dpcomp_buf(csize8)
	common /ccm_cx8/ dpcomp_buf
	
	real(b8) :: dp_buf(bsize8)
	common /ccm_rx8/ dp_buf
	
	complex(c4) :: comp_buf(csize4)
	common /ccm_cx4/ comp_buf
	
	real(b4) :: sp_buf(bsize4)
	common /ccm_rx4/ sp_buf

	integer(def_int) :: in_buf(isize4)
	common /ccm_ix4/ in_buf
		
	logical :: logical_buf(lsize4)
	common /ccm_lx4/ logical_buf

	character(len=1) :: character_buf(asize4)
	common /ccm_ax4/ character_buf

!
	integer :: sync_block
	common /ccm_p2p/ sync_block(-1:maxprocs*2)
	
	integer(selected_int_kind(18)) :: mycray(4,maxprocs),myflag(0:maxprocs)
    common /ccm_mysgi/ mycray,myflag

	integer, parameter :: MPI_STATUS_SIZE=1
	integer, parameter :: mpi_real=-1
	integer, parameter :: mpi_double_precision=-1
	integer, parameter :: mpi_integer=-1
	integer, parameter :: mpi_complex=-1
	integer, parameter :: mpi_logical=-1
	save 
! end of stuff unique to shmem

 contains
     function ccm_time()
        implicit none
        real(b8) :: ccm_time,tmp
        integer,parameter :: norm(13)=(/  &   
               0, 2678400, 5097600, 7776000,10368000,13046400,&
        15638400,18316800,20995200,23587200,26265600,28857600,31536000/)
        integer,parameter :: leap(13)=(/  &   
               0, 2678400, 5184000, 7862400,10454400,13132800,&
        15724800,18403200,21081600,23673600,26352000,28944000,31622400/)
        integer :: values(8),m,sec
        call date_and_time(values=values)
        if(mod(values(1),4) .eq. 0)then
           m=leap(values(2))
        else
           m=norm(values(2))
        endif
        sec=((values(3)*24+values(5))*60+values(6))*60+values(7)
        sec=sec-60*values(4)
        tmp=real(m,b8)+real(sec,b8)+real(values(8),b8)/1000.0_b8
        !write(*,*)"vals ",values
        if(values(1) .ne. ccm_start_time(1))then
            if(mod(ccm_start_time(1),4) .eq. 0)then
                tmp=tmp+real(leap(13),b8)
            else
                tmp=tmp+real(norm(13),b8)
            endif
        endif
        ccm_time=tmp
    end function
    subroutine ccm_testing(get_test,set_test)
        implicit none
        integer,optional :: get_test,set_test
        if(present(get_test))get_test=do_tests
        if(present(set_test))do_tests=set_test
    end subroutine

end module

